

#include <windows.h>
#include <winsvc.h>
#include <stdio.h>
#include <string>

#include "shoutAkiRa.h"
#include "httpserver.h"
#include "httpstatus.h"
#include "buffer.h"
#include "html.h"
#include "encoding/jstring.h"



#define SERVICE_NAME "shoutAkiRa"
#define DISPLAY_NAME "shoutAkiRa streaming data source service"






void install_service(char const *path)
{
	int err = 0;

	SC_HANDLE scm = 0;
	SC_HANDLE srv = 0;
	scm = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS);
	if (!scm) {
		err = GetLastError();
		goto done;
	}
	srv = CreateService(scm, SERVICE_NAME, DISPLAY_NAME, SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_AUTO_START, SERVICE_ERROR_NORMAL, path, 0, 0, 0, 0, 0);
	if (!srv) {
		err = GetLastError();
		goto done;
	}

	err = 0;
done:;
	if (srv) {
		CloseServiceHandle(srv);
	}
	if (scm) {
		CloseServiceHandle(scm);
	}

	if (err != 0) {
		fprintf(stderr, "error 0x%08x in install_service()\n", err);
	}
}

void uninstall_service()
{
	int err = 0;

	SC_HANDLE scm = 0;
	SC_HANDLE srv = 0;
	scm = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS);
	if (!scm) {
		err = GetLastError();
		goto done;
	}
	srv = OpenService(scm, SERVICE_NAME, DELETE);
	if (!srv) {
		err = GetLastError();
		goto done;
	} else {
		if (!DeleteService(srv)) {
			err = GetLastError();
			goto done;
		}
	}

	err = 0;
done:;
	if (srv) {
		CloseServiceHandle(srv);
	}
	if (scm) {
		CloseServiceHandle(scm);
	}

	if (err != 0) {
		fprintf(stderr, "error 0x%08x in install_service()\n", err);
	}
}

bool start_service()
{
	bool success = false;

	SC_HANDLE scm = 0;
	SC_HANDLE srv = 0;
	SERVICE_STATUS st;
	memset(&st, 0, sizeof(st));

	scm = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS);
	if (!scm) {
		goto done;
	}
	srv = OpenService(scm, SERVICE_NAME, SERVICE_QUERY_STATUS | SERVICE_START);
	if (!srv) {
		goto done;
	}
	if (!StartService(srv, 0, 0)) {
		goto done;
	}

	for (int i = 0; i < 10; i++) {
		if (st.dwCurrentState == SERVICE_RUNNING) {
			break;
		}
		Sleep(100);
		if (!QueryServiceStatus(srv, &st)) {
			goto done;
		}
	}

	success = true;

done:;
	if (srv) {
		CloseServiceHandle(srv);
	}
	if (scm) {
		CloseServiceHandle(scm);
	}

	return success;
}

bool stop_service()
{
	bool success = false;

	SC_HANDLE scm = 0;
	SC_HANDLE srv = 0;
	SERVICE_STATUS st;
	memset(&st, 0, sizeof(st));

	scm = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS);
	if (!scm) {
		goto done;
	}
	srv = OpenService(scm, SERVICE_NAME, SERVICE_QUERY_STATUS | SERVICE_STOP);
	if (!srv) {
		goto done;
	}
	if (!ControlService(srv, SERVICE_CONTROL_STOP, &st)) {
		goto done;
	}

	for (int i = 0; i < 10; i++) {
		if (st.dwCurrentState == SERVICE_RUNNING) {
			break;
		}
		Sleep(100);
		if (!QueryServiceStatus(srv, &st)) {
			goto done;
		}
	}

	success = true;

done:;
	if (srv) {
		CloseServiceHandle(srv);
	}
	if (scm) {
		CloseServiceHandle(scm);
	}

	return success;
}




SERVICE_STATUS_HANDLE service_status_handle;
SERVICE_STATUS service_status = {
	SERVICE_WIN32_OWN_PROCESS,
	SERVICE_START_PENDING,
	SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN | SERVICE_ACCEPT_POWEREVENT,
	NO_ERROR,
	NO_ERROR,
	0,
	0
};

DWORD WINAPI HandlerEx(DWORD dwControl, DWORD dwEventType, LPVOID lpEventData, LPVOID lpContext)
{
	switch (dwControl) {
	case SERVICE_CONTROL_STOP:
		service_status.dwCurrentState = SERVICE_STOP_PENDING;
		service_status.dwWin32ExitCode = 0;
		service_status.dwCheckPoint = 0;
		service_status.dwWaitHint = 0;
		SetServiceStatus(service_status_handle, &service_status);
		break;
	}
	return NO_ERROR;
}

void WINAPI ServiceMain(DWORD ac, char **av)
{
	service_status_handle = RegisterServiceCtrlHandlerEx(SERVICE_NAME, HandlerEx, 0);
	if (!service_status_handle) {
		return;
	}

	ShoutShark shark;

	shark.Main(true);
}

void show_information()
{
	int status = -1;

	SC_HANDLE scm = 0;
	SC_HANDLE srv = 0;
	SERVICE_STATUS st;
	memset(&st, 0, sizeof(st));

	scm = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS);
	if (!scm) {
		goto done;
	}
	srv = OpenService(scm, SERVICE_NAME, SERVICE_QUERY_STATUS);
	if (!srv) {
		goto done;
	}

	if (!QueryServiceStatus(srv, &st)) {
		goto done;
	}

	status = st.dwCurrentState;

done:;
	if (srv) {
		CloseServiceHandle(srv);
	}
	if (scm) {
		CloseServiceHandle(scm);
	}

	char const *text = "unknown";

	switch (status) {
	case SERVICE_STOPPED:              text = "stopped";            break;
	case SERVICE_START_PENDING:        text = "start pending";      break;
	case SERVICE_STOP_PENDING:         text = "stop pending";       break;
	case SERVICE_RUNNING:              text = "running";            break;
	case SERVICE_CONTINUE_PENDING:     text = "continue pending";   break;
	case SERVICE_PAUSE_PENDING:        text = "pause pending";      break;
	case SERVICE_PAUSED:               text = "paused";             break;
	}

	printf("shoutakira service: %s\n", text);
}

void show_version()
{
	printf("shoutAkiRa v0.0, the streaming data source server for SHOUTcast.\n");
	printf("Copyright (C) 2010 S.Fuchita. http://www.soramimi.jp/radio/\n");
}

bool is_service_start_pending()
{
	bool result = false;
	SC_HANDLE scm = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS);
	if (scm) {
		SC_HANDLE srv = OpenService(scm, SERVICE_NAME, SERVICE_QUERY_STATUS);
		if (srv) {
			SERVICE_STATUS st;
			memset(&st, 0, sizeof(st));
			if (QueryServiceStatus(srv, &st)) {
				if (st.dwCurrentState == SERVICE_START_PENDING) {
					result = true;
				}
			}
			CloseServiceHandle(srv);
		}
		CloseServiceHandle(scm);
	}
	return result;
}

std::string get_module_path()
{
	char tmp[MAX_PATH];
	GetModuleFileName(0, tmp, MAX_PATH);
	return tmp;
}

int main(int argc, char **argv)
{
	if (is_service_start_pending()) {
		SERVICE_TABLE_ENTRY ent[] = { { SERVICE_NAME, (LPSERVICE_MAIN_FUNCTION)ServiceMain }, { 0, 0 }, };
		if (!StartServiceCtrlDispatcher(ent)) {
			return 1;
		}
		return 0;
	}

	if (argc == 1) {
		show_information();
		return 0;
	}

	if (argc != 2) {
		fprintf(stderr, "argument count incorrect\n");
		return 1;
	}

	if (strcmp(argv[1], "version") == 0) {
		show_version();
		return 0;
	}

	if (strcmp(argv[1], "run") == 0) {
		ShoutShark shark;
		shark.Main(false);
		return 0;
	}

	if (strcmp(argv[1], "install") == 0) {
		install_service(get_module_path().c_str());
		return 0;
	}

	if (strcmp(argv[1], "uninstall") == 0) {
		uninstall_service();
		return 0;
	}

	if (strcmp(argv[1], "start") == 0) {
		start_service();
		return 0;
	}

	if (strcmp(argv[1], "stop") == 0) {
		stop_service();
		return 0;
	}

	return 0;
}


